@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2013 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
@@ -22,13 +22,9 @@
 import unittest2
 import datetime
 import logging
-from stalker import db
-from stalker import config
-
-defaults = config.Config()
 
 from stalker.db.session import DBSession
-from stalker import (Group, Department, Project, Repository,
+from stalker import (db, defaults, Group, Department, Project, Repository,
                      Sequence, Status, StatusList, Task, Type, User, Version,
                      Ticket, Vacation)
 
@@ -45,13 +41,12 @@ class UserTest(unittest2.TestCase):
         """set up the test for class
         """
         DBSession.remove()
-        DBSession.configure(extension=None)
 
     @classmethod
     def tearDownClass(cls):
         """clean up the test
         """
-        DBSession.configure(extension=None)
+        DBSession.remove()
 
     def setUp(self):
         """setup the test
@@ -173,16 +168,8 @@ class UserTest(unittest2.TestCase):
         ])
 
         # a task status list
-        self.task_status_list = StatusList(
-            name="Task Status List",
-            statuses=[
-                self.test_status1,
-                self.test_status2,
-                self.test_status3,
-                self.test_status4
-            ],
-            target_entity_type=Task
-        )
+        self.task_status_list = StatusList.query\
+            .filter_by(target_entity_type='Task').first()
 
         # a couple of tasks
         self.test_task1 = Task(
@@ -216,37 +203,21 @@ class UserTest(unittest2.TestCase):
             self.test_task4
         ])
 
-        # a couple of versions
-        # a version status list
-        self.version_status_list = StatusList(
-            name="Version Status List",
-            statuses=[
-                self.test_status1,
-                self.test_status2,
-                self.test_status3,
-                self.test_status4
-            ],
-            target_entity_type=Version
-        )
-
         # for task1
         self.test_version1 = Version(
             task=self.test_task1,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version1)
 
         self.test_version2 = Version(
             task=self.test_task1,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version2)
 
         self.test_version3 = Version(
             task=self.test_task1,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version3)
@@ -254,21 +225,18 @@ class UserTest(unittest2.TestCase):
         # for task2
         self.test_version4 = Version(
             task=self.test_task2,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version4)
 
         self.test_version5 = Version(
             task=self.test_task2,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version5)
 
         self.test_version6 = Version(
             task=self.test_task2,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version6)
@@ -276,21 +244,18 @@ class UserTest(unittest2.TestCase):
         # for task3
         self.test_version7 = Version(
             task=self.test_task3,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version7)
 
         self.test_version8 = Version(
             task=self.test_task3,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version8)
 
         self.test_version9 = Version(
             task=self.test_task3,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version9)
@@ -298,21 +263,18 @@ class UserTest(unittest2.TestCase):
         # for task4
         self.test_version10 = Version(
             task=self.test_task4,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version10)
 
         self.test_version11 = Version(
             task=self.test_task4,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version11)
 
         self.test_version12 = Version(
             task=self.test_task4,
-            status_list=self.version_status_list,
             full_path='some/path'
         )
         DBSession.add(self.test_version12)
@@ -407,16 +369,8 @@ class UserTest(unittest2.TestCase):
         # *********************************************************************
 
         # a status list for sequence
-        self.sequence_status_list = StatusList(
-            name="Sequence Status List",
-            statuses=[
-                self.test_status1,
-                self.test_status2,
-                self.test_status3,
-                self.test_status4
-            ],
-            target_entity_type=Sequence
-        )
+        self.sequence_status_list = StatusList.query\
+            .filter_by(target_entity_type='Sequence').first()
 
         # a couple of sequences
         self.test_sequence1 = Sequence(
@@ -472,18 +426,11 @@ class UserTest(unittest2.TestCase):
             'description': 'this is a test user',
             'password': 'hidden',
             'email': 'eoyilmaz@fake.com',
-            'department': self.test_department1,
+            'departments': [self.test_department1],
             'groups': [self.test_group1,
                        self.test_group2],
-            'projects_lead': [self.test_project1,
-                              self.test_project2],
-            'sequences_lead': [self.test_sequence1,
-                               self.test_sequence2,
-                               self.test_sequence3,
-                               self.test_sequence4],
             'created_by': self.test_admin,
             'updated_by': self.test_admin,
-            'last_login': None
         }
 
         # create a proper user object
@@ -567,7 +514,7 @@ class UserTest(unittest2.TestCase):
             "@",
             "eoyilmaz@",
             "eoyilmaz@some.compony@com",
-        ]
+            ]
         # any of these email values should raise a ValueError
         for value in test_values:
             self.assertRaises(
@@ -578,7 +525,24 @@ class UserTest(unittest2.TestCase):
                 value
             )
 
-    def test_email_attribute_works_properly(self):
+    def test_email_argument_should_be_a_unique_value(self):
+        """testing if the email argument should be a unique value
+        """
+        # this test should include a database
+        test_email = "test@email.com"
+        self.kwargs['login'] = 'test_user1'
+        self.kwargs['email'] = test_email
+        user1 = User(**self.kwargs)
+        DBSession.add(user1)
+        DBSession.commit()
+
+        self.kwargs['login'] = 'test_user2'
+        user2 = User(**self.kwargs)
+        DBSession.add(user2)
+
+        self.assertRaises(Exception, DBSession.commit)
+
+    def test_email_attribute_is_working_properly(self):
         """testing if email attribute works properly
         """
         test_email = "eoyilmaz@somemail.com"
@@ -653,7 +617,7 @@ class UserTest(unittest2.TestCase):
             (" eRkAn", "erkan"),
             (" eRkan ozGur", "erkanozgur"),
             ("213 e.ozgur", "eozgur"),
-        ]
+            ]
 
         for valuePair in test_values:
             # set the input and expect the expected output
@@ -688,6 +652,24 @@ class UserTest(unittest2.TestCase):
                 valuePair[1]
             )
 
+    def test_login_argument_should_be_a_unique_value(self):
+        """testing if the login argument should be a unique value
+        """
+        # this test should include a database
+        test_login = 'test_user1'
+        self.kwargs['login'] = test_login
+        self.kwargs['email'] = "test1@email.com"
+        user1 = User(**self.kwargs)
+        DBSession.add(user1)
+        DBSession.commit()
+
+        self.kwargs['email'] = "test2@email.com"
+
+        user2 = User(**self.kwargs)
+        DBSession.add(user2)
+
+        self.assertRaises(Exception, DBSession.commit)
+
     def test_login_argument_is_working_properly(self):
         """testing if the login argument is working properly
         """
@@ -700,14 +682,6 @@ class UserTest(unittest2.TestCase):
         self.test_user.login = test_value
         self.assertEqual(self.test_user.login, test_value)
 
-    def test_last_login_argument_accepts_None(self):
-        """testing if nothing happens when the last login argument is set to
-        None
-        """
-        self.kwargs["last_login"] = None
-        # nothing should happen
-        a_new_user = User(**self.kwargs)
-
     def test_last_login_attribute_None(self):
         """testing if nothing happens when the last login attribute is set to
         None
@@ -715,25 +689,6 @@ class UserTest(unittest2.TestCase):
         # nothing should happen
         self.test_user.last_login = None
 
-    def test_last_login_argument_accepts_datetime_instance(self):
-        """testing if a nothing happens when tried to set the last_login
-        to a datetime.datetime instance
-        """
-        self.kwargs["last_login"] = datetime.datetime.now()
-        # nothing should happen
-        a_new_user = User(**self.kwargs)
-
-    def test_last_login_argument_accepts_only_datetime_instance_or_None(self):
-        """testing if a TypeError will be raised for values other than a
-        datetime.datetime instances or None tried to be set to last_login
-        argument
-        """
-        test_values = [1, 2.3, "login time", ["last login time"],
-                       {"a last": "login time"}]
-        for test_value in test_values:
-            self.kwargs["last_login"] = test_value
-            self.assertRaises(TypeError, User, **self.kwargs)
-
     def test_last_login_attribute_accepts_only_datetime_instance_or_None(self):
         """testing if a TypeError will be raised for values other than
         datetime.datetime instances tried to be assigned to last_login
@@ -1008,14 +963,6 @@ class UserTest(unittest2.TestCase):
         self.assertIn(self.test_user, self.test_project2.users)
         self.assertIn(self.test_user, self.test_project3.users)
 
-    def test_projects_lead_argument_None(self):
-        """testing if the projects_lead attribute will be an empty list when
-        the projects_lead attribute is None
-        """
-        self.kwargs["projects_lead"] = None
-        new_user = User(**self.kwargs)
-        self.assertEqual(new_user.projects_lead, [])
-
     def test_projects_lead_attribute_None(self):
         """testing if a TypeError will be raised when the project_lead
         attribute is set to None
@@ -1023,13 +970,6 @@ class UserTest(unittest2.TestCase):
         self.assertRaises(TypeError, setattr, self.test_user, "projects_lead",
                           None)
 
-    def test_projects_lead_argument_accepts_empty_list(self):
-        """testing if projects_lead argument accepts an empty list
-        """
-        self.kwargs["projects_lead"] = []
-        # this should work without any problems
-        self.test_user = User(**self.kwargs)
-
     def test_projects_lead_attribute_accepts_empty_list(self):
         """testing if projects_lead attribute accepts an empty list
         """
@@ -1037,26 +977,6 @@ class UserTest(unittest2.TestCase):
         # this should work without any problem
         self.test_user.projects_lead = []
 
-
-    def test_projects_lead_argument_accepts_only_list(self):
-        """testing if a TypeError will be raised when trying to assign a list
-        of other objects than a list of Project objects to the
-        projects_lead argument
-        """
-        test_values = ["a project", 123123, {}, 12.2132]
-        for test_value in test_values:
-            self.kwargs["projects_lead"] = test_value
-            self.assertRaises(TypeError, User, **self.kwargs)
-
-    def test_projects_lead_argument_accepts_only_lists_of_project_obj(self):
-        """testing if a TypeError will be raised when trying to assign a list
-        of other objects than a list of Project objects to the
-        projects_lead argument
-        """
-        test_value = ["a project", 123123, [], {}, 12.2132]
-        self.kwargs["projects_lead"] = test_value
-        self.assertRaises(TypeError, User, **self.kwargs)
-
     def test_projects_lead_attribute_accepts_only_lists(self):
         """testing if a TypeError will be raised when trying to assign a list
         of other objects than a list of Project objects to the
@@ -1106,38 +1026,12 @@ class UserTest(unittest2.TestCase):
             0
         )
 
-        # __setitem__
-        self.assertRaises(
-            TypeError,
-            self.test_user.projects_lead.__setitem__,
-            0,
-            0
-        )
-
-    def test_tasks_argument_None(self):
-        """testing if the tasks attribute will be an empty list when the tasks
-        argument is given as None
-        """
-        self.kwargs["tasks"] = None
-        new_user = User(**self.kwargs)
-        self.assertEqual(new_user.tasks, [])
-
     def test_tasks_attribute_None(self):
         """testing if a TypeError will be raised when the tasks attribute is
         set to None
         """
         self.assertRaises(TypeError, setattr, self.test_user, "tasks", None)
 
-    def test_tasks_argument_accepts_only_list_of_task_objects(self):
-        """testing if a TypeError will be raised when trying to assign
-        anything other than a list of task objects to the tasks argument
-        """
-        test_values = [12312, 1233244.2341, ["aTask1", "aTask2"], "a_task"]
-
-        for test_value in test_values:
-            self.kwargs["tasks"] = test_value
-            self.assertRaises(TypeError, User, **self.kwargs)
-
     def test_tasks_attribute_accepts_only_list_of_task_objects(self):
         """testing if a TypeError will be raised when trying to assign
         anything other than a list of task objects to the tasks argument
@@ -1153,15 +1047,6 @@ class UserTest(unittest2.TestCase):
                 test_value
             )
 
-    def test_tasks_argument_accepts_an_empty_list(self):
-        """testing if nothing happens when trying to assign an empty list to
-        tasks argument
-        """
-        self.kwargs["tasks"] = []
-
-        # this should work without any error
-        aUserObj = User(**self.kwargs)
-
     def test_tasks_attribute_accepts_an_empty_list(self):
         """testing if nothing happens when trying to assign an empty list to
         tasks attribute
@@ -1227,7 +1112,7 @@ class UserTest(unittest2.TestCase):
         """
         self.assertEqual(
             self.test_user.__repr__(),
-            "<User (%s ('%s'))>" % (
+            "<%s ('%s') (User)>" % (
                 self.test_user.name,
                 self.test_user.login)
         )
@@ -1245,8 +1130,9 @@ class UserTest(unittest2.TestCase):
     def test_tickets_attribute_is_read_only(self):
         """testing if the User.tickets attribute is a read only attribute
         """
-        self.assertRaises(AttributeError, setattr, self.test_user, 'tickets',
-                          [])
+        self.assertRaises(
+            AttributeError, setattr, self.test_user, 'tickets', []
+        )
 
     def test_open_tickets_attribute_is_read_only(self):
         """testing if the User.open_tickets attribute is a read only attribute
@@ -1356,8 +1242,8 @@ class UserTest(unittest2.TestCase):
     def test_to_tjp_is_working_properly(self):
         """testing if the to_tjp property is working properly
         """
-        expected_tjp = 'resource User_68 "Erkan Ozgur Yilmaz"'
-        self.assertEqual(self.test_user.to_tjp, expected_tjp)
+        expected_tjp = 'resource User_82 "Erkan Ozgur Yilmaz"'
+        self.assertEqual(expected_tjp, self.test_user.to_tjp)
 
     def test_to_tjp_is_working_properly_for_a_user_with_vacations(self):
         """testing if the to_tjp property is working properly for a user with
@@ -1383,16 +1269,16 @@ class UserTest(unittest2.TestCase):
             end=datetime.datetime(2013, 7, 15, 0, 0)
         )
 
-        expected_tjp = """resource User_68 "Erkan Ozgur Yilmaz" {
+        expected_tjp = """resource User_82 "Erkan Ozgur Yilmaz" {
             vacation 2013-06-07-00:00:00 - 2013-06-21-00:00:00
             vacation 2013-07-01-00:00:00 - 2013-07-15-00:00:00
             }"""
-        print expected_tjp
-        print '---------------'
-        print self.test_user.to_tjp
+        # print expected_tjp
+        # print '---------------'
+        # print self.test_user.to_tjp
         self.assertEqual(
-            self.test_user.to_tjp,
-            expected_tjp
+            expected_tjp,
+            self.test_user.to_tjp
         )
 
     def test_vacations_attribute_is_set_to_None(self):
